library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.math_pack.all;

--------------ENTITY----------------------------------------
entity harderr is
    generic(
        PRM: integer	 
	  );
    port( 
        clk         : in std_logic;
        rst         : in std_logic;                      -- globar reset     
        rrerr_in    : in std_logic_vector(PRM-1 downto 0); -- PRM index
        refresh     : in std_logic;                      -- refresh of last Err register
        clear       : in std_logic;                      -- clear actual Err register
        enable      : in std_logic;                      -- enable detection of Hard Err
        harderr     : out std_logic                      -- hard '1'/'0'
    ); 
end harderr;

-----------------------------------------------------------------------------
--------------ARCHITECTURE---------------------------------------------------
-----------------------------------------------------------------------------
architecture behav of harderr is

signal reglast : std_logic_vector(PRM-1 downto 0); -- error registers
signal prm_rec_reg : std_logic_vector(PRM-1 downto 0); -- error registers

begin

--============Reconfigurated PRM Register============================
    process(clk, rst)
    begin
        if (rst = '1') then
            prm_rec_reg <= (others => '0');

        elsif rising_edge(clk) then
            if(clear = '1') then
                prm_rec_reg <= (others => '0');
            else
                prm_rec_reg <= prm_rec_reg or rrerr_in;      
            end if;
        end if;
    end process;
    

--============Error Register Last===================================
    process(clk, rst)
    begin
        if (rst = '1') then
            reglast <= (others => '0');
        elsif rising_edge(clk) then
            if (refresh = '1') then
                reglast <= prm_rec_reg; 
            end if;
        end if;
    end process;
-------------------------------------------------------------------


--============Hard Error Detection=================================   
    
    harderr <= '0' when ( ( (reglast and rrerr_in) = conv_std_logic_vector(0, PRM) ) or enable = '0') else '1'; --harderr_pom(PRM);
-------------------------------------------------------------------- 
end behav;
