--Unit for computation of the parity of the frame

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity frame_parity is
   port(
      clk : in std_logic;
      rst : in std_logic;
      start : in std_logic;
      data_in : in std_logic_vector(7 downto 0);
      data_valid : in std_logic;
      parity_ok : out std_logic
   );
end frame_parity;

architecture behv of frame_parity is

type states is (
  INIT,
	BYTE1,
	BYTE10,
	BYTE2,
	BYTE20,
	BYTE3,
	BYTE30,
  BYTE4,
	DATA_WORD_READ
);
   signal state : states := INIT;
	signal nextstate :states := INIT;
   signal reset : std_logic;
   signal data_word : std_logic_vector(31 downto 0);
   signal data_word_valid : std_logic;
   signal parity_reg : std_logic;
   signal partree1 : std_logic_vector(6 downto 0);
   signal partree2 : std_logic_vector(1 downto 0);
   signal enable_byte1 : std_logic;
	 signal enable_byte2 : std_logic;
	 signal enable_byte3 : std_logic;
	 signal enable_byte4 : std_logic;
	 
begin
    reset <= rst or start;  --I am going to reset parity register with reset or at the start of frame

    state_current : process(clk,reset,state)
	  begin
        if (reset='1') then
            state <= INIT;
			elsif rising_edge(clk) then
            state <= nextstate;
		
			  end if;
    end process;
     

    state_machine : process(state,data_valid)
	 begin
	     enable_byte1 <= '0';
		  enable_byte2 <= '0';
		  enable_byte3 <= '0';
		  enable_byte4 <= '0';
		  data_word_valid <= '0';
        nextstate <= INIT;
		  
		  case state is
		  
			   when INIT =>
				    if (data_valid = '1') then
		              nextstate <= BYTE1;
					 else
					     nextstate <= INIT;
		          end if;
					 
            when BYTE1 =>
                enable_byte1 <= '1';
                nextstate <= BYTE10;
            
            when BYTE10 =>
                if (data_valid = '1') then
		              nextstate <= BYTE2;
					 else
					     nextstate <= BYTE10;
                end if;
            
            when BYTE2 =>
			       enable_byte2 <= '1';
		          nextstate <= BYTE20;
            
            when BYTE20 =>
			       if (data_valid = '1') then
		              nextstate <= BYTE3;
					 else
					     nextstate <= BYTE20;
                end if;
                
            when BYTE3 =>
			       enable_byte3 <= '1';
		          nextstate <= BYTE30;
                           
            when BYTE30 =>
			       if (data_valid = '1') then
		              nextstate <= BYTE4;
					 else
					     nextstate <= BYTE30;
                end if;
                
            when BYTE4 =>
                enable_byte4 <= '1';
			       nextstate <= DATA_WORD_READ;
            
            when DATA_WORD_READ =>
                data_word_valid <= '1';
					 if(data_valid = '1') then
                    nextstate <= BYTE1;
					 else
					     nextstate <= DATA_WORD_READ;
                end if;
			   
				when others => nextstate <= INIT;
            
				end case;
    end process;

    parity_comp: for N in 0 to 5 generate
        partree1(N) <= data_word(N*5+0) xor data_word(N*5+1) xor data_word(N*5+2) xor data_word(N*5+3) xor data_word(N*5+4);
    end generate;
    partree1(6) <= data_word(30) xor data_word(31);

    partree2(0) <= partree1(0) xor partree1(1) xor partree1(2) xor partree1(3) xor partree1(4);
    partree2(1) <= partree1(5) xor partree1(6) xor partree2(0) xor parity_reg;

    par_gen: process(clk,reset,data_word_valid)
    begin
        if(reset = '1') then
            parity_reg <= '0';
        elsif rising_edge(clk) then
            if(data_word_valid = '1') and (data_valid = '1') then
                parity_reg <= partree2(1);
            end if;
        end if;
    end process;


    data_word_register: process(clk, reset)
    begin
        if(reset = '1') then
            data_word <= (others => '0');
        elsif rising_edge(clk) then
            if(enable_byte1 = '1') then
                data_word(31 downto 24) <= data_in(7 downto 0);
            end if; 
            if(enable_byte2 = '1') then
                data_word(23 downto 16) <= data_in(7 downto 0);
            end if; 
            if(enable_byte3 = '1') then
                data_word(15 downto 8) <= data_in(7 downto 0);
            end if; 
            if(enable_byte4 = '1') then
                data_word(7 downto 0) <= data_in(7 downto 0);
            end if;    
        end if;
    end process;

    parity_ok <= not parity_reg;
end behv;

   
   
